// -*- Mode: C++; -*-
#ifndef __mixer_h
#define __mixer_h
#include <sys/soundcard.h>
#include <stdlib.h>
#include <unistd.h>
#include <string>

#include "xqmixer/helpers/smart_pointer.h"

//forward declaration
class mixer_dev;

class mixer:public ref_count
{
    friend class mixer_dev;

protected:
    int mixer_fd;
    int devmask;
    int recmask;
    int recsrc;
    int stereodev;
    static const char*names[SOUND_MIXER_NRDEVICES];
    int exclusiv_record;
    std::string DevName;

public:
    mixer();
    virtual ~mixer();
    int get_mixer_fd(){return mixer_fd;}
    int open_mixer(const char*devname=NULL);
    void close_mixer(){if (mixer_fd) close(mixer_fd); mixer_fd = 0;}
    int get_devmask(){return  devmask;}
    int get_recmask(){return recmask;}
    int get_recsrc(){return recsrc;}
    int get_stereodev(){return stereodev;}
    int is_valid_dev(int dev);
    int is_exclusiv(){return exclusiv_record;}
    void set_recmask();
    const char*get_name()const{return DevName.c_str();}
};

typedef smart_pointer<mixer> mixerP;

class mixer_dev {

private:
    int devnumber;
    int balance;
    int value;
    const char*Name;
    mixerP use_mixer;

public:
    virtual ~mixer_dev(){};
    void set_balance(int nBalance);
    const int get_balance()const{return balance;}
    void set_value(int nValue);
    const int get_value()const{return value;}
    void read_values();
    mixer_dev(int nDev,mixer*use_mix);
    mixer_dev(){use_mixer = 0;Name = "";}
    const char*get_name()const{return Name;}
    const int is_stereo()const;
    const int is_recordsrc()const;
    void set_record(int On);
    const int get_record()const;
};

#endif
