#include <qapp.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qstring.h>
#include <qmsgbox.h>
#include <qobjcoll.h>
#include <qfiledlg.h>
#include <errno.h>
#include "helpers/helpers.h"
#include "options.h"
#include "mwidget.h"
#include <qpushbutton.h>
#include <iostream>

MixerWidget::MixerWidget(QWidget*parent, const char*geraet, const char*name)
    :QWidget(parent,name)
{
    MixerWidgetLayout = new QHBoxLayout(this,0,0,"MixerWidgetLayout");
    IntList = 0;
    QString d;

    Options = new options();
    Options->load_options();

    int i = 0;
    use_device = new mixer;
    if (geraet && strlen(geraet) > 0) {
	i = use_device->open_mixer(geraet);
    } else {
	//open default the mixer device
	i = use_device->open_mixer();
    }
    if (i < 1) {
	QString er = "Error open Mixer:\n";
	er += strerror(errno);
	er += "\n";
	er += geraet;
	QLabel * Fehler = new QLabel(er,this,"error");
	MixerWidgetLayout->addWidget(Fehler);
    } else {
        Mixer*mix;
        IntList = new QList<Mixer>;
        CHECK_PTR(IntList);
        //IntList->setAutoDelete(TRUE);
        //stepping the devices
        for (i=0; i < SOUND_MIXER_NRDEVICES;i++) {
            //does it exists?
            if (use_device->is_valid_dev(i)) {
                mix = new  Mixer(use_device,this,0,i);
                CHECK_PTR(mix);
                //mix->setMinimumSize(40,90);
                MixerWidgetLayout->addWidget(mix);
                IntList->append(mix);
                if (Options->get_load_on_start() ) {
                    mix->readvalues(Options,Options->get_load_only_layout());
                    if (!Options->get_load_only_vals()) {
                        if (!Options->get_all_display_on())
                            mix->set_last_display();
                    }
                }
            }
        }
        if (Options->get_all_display_on())
            toggle_value_display(0);
    }
}

MixerWidget::~MixerWidget()
{
    //    mixer::close_mixer();
    if (IntList)
	delete IntList;
}

void MixerWidget::save_vals()
{
    if (!IntList || !Options)
	return;
    Mixer*mix;
    for ( mix=IntList->first(); mix != 0; mix=IntList->next() )
	mix->writevalues(Options);        
}

void MixerWidget::save_mixer()
{
    if (!Options || !IntList)
	return;
    QString file = QFileDialog::getSaveFileName(0,"*.mxr");
    if (file.isNull())
	return;
    QString o_file = Options->getfile().c_str();
    Options->setfile(file);
    Mixer*mix;
    for ( mix=IntList->first(); mix != 0; mix=IntList->next() )
	mix->writevalues(Options);    
    Options->setfile(o_file);
}

void MixerWidget::read_mixer()
{
    if (!Options || !IntList)
	return;
    QString file = QFileDialog::getOpenFileName(0,"*.mxr");
    if (file.isNull())
	return;
    QString o_file = Options->getfile().c_str();
    Options->setfile(file);
    Mixer*mix;
    for ( mix=IntList->first(); mix != 0; mix=IntList->next() )
	mix->readvalues(Options,0);
    Options->setfile(o_file);
}

void MixerWidget::toggle_value_display(int last_act)
{
    if (!IntList)
	return;
    for (Mixer* mix=IntList->first(); mix != 0; mix=IntList->next() ) {
	if (!last_act) 
	    mix->display_on();
	else
	    mix->display_off();
    }
}

/**  */
const char* MixerWidget::get_dev_name()const
{
    if (use_device)
	return use_device->get_name();
    else
	return 0;
}

void MixerWidget::resizeEvent(QResizeEvent*e)
{
    if (!IntList || IntList->count()==0)
        return QWidget::resizeEvent(e);
    QSize s = e->size();
    int x = s.width()/IntList->count();
    int y = s.height();
    QWidget*w;
    int i = 0;
    setUpdatesEnabled(FALSE);
    for (w=IntList->first();w;w=IntList->next()) {
        w->setGeometry(QRect(i*x,0,x,y));
        ++i;
    }
    setUpdatesEnabled(TRUE);
    repaint();
}
